@@ -1,7 +1,7 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-1998 by Brian V. Smith
+ * Parts Copyright (c) 1989-2000 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  * Parts Copyright (c) 1995 by C. Blanc and C. Schlick
  *
@@ -30,5 +30,11 @@ void		draw_compoundelements();
 
 void            draw_spline();
 void            quick_draw_spline();
+
+/* curve routine needed by arc() and show_boxradius() */
+
+void curve(Window, int, int, int, int, Boolean, Boolean,
+		int, int, int, int, int, int, int, int, 
+		float, int, int, int, int);
  
 #endif /* U_DRAW_H */
