@@ -1,7 +1,7 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1985-1988 by Supoj Sutanthavibul
- * Parts Copyright (c) 1989-1998 by Brian V. Smith
+ * Parts Copyright (c) 1989-2000 by Brian V. Smith
  * Parts Copyright (c) 1991 by Paul King
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
@@ -128,5 +128,5 @@ my_quit(w, event, params, num_params)
 	return;
     }
     /* quit after asking whether user wants to save figure */
-    quit(w);
+    quit(w, 0, 0);
 }
