@@ -1,7 +1,7 @@
 /*
  * FIG : Facility for Interactive Generation of figures
  * Copyright (c) 1991 by Paul King
- * Parts Copyright (c) 1989-1998 by Brian V. Smith
+ * Parts Copyright (c) 1989-2000 by Brian V. Smith
  *
  * Any party obtaining a copy of these files is granted, free of charge, a
  * full and unrestricted irrevocable, world-wide, paid up, royalty-free,
@@ -34,13 +34,19 @@ Widget		print_orient_panel;
 Widget		print_just_panel;
 Widget		print_papersize_panel;
 Widget		print_multiple_panel;
+Widget		printer_menu_button;
 Widget		print_mag_text;
+Widget		print_background_panel;
 void		print_update_figure_size();
 
 /* LOCAL */
 
 DeclareStaticArgs(15);
 
+static char    *printer_names[200];	/* allow 200 printers */
+static int	parse_printcap();
+static int	numprinters;
+
 static void	orient_select();
 static Widget	orient_menu, orient_lab;
 
@@ -50,12 +56,19 @@ static Widget	just_menu, just_lab;
 static void	papersize_select();
 static Widget	papersize_menu, papersize_lab;
 
+static void	background_select();
+static Widget	background_lab, background_menu;
+
 static void	multiple_select();
 static Widget	multiple_menu, multiple_lab;
 
+static void	printer_select();
+static Widget	printer_menu;
+
 static Widget	print_panel, dismiss, print, 
-		printer_text, param_text, printer_lab, param_lab, clear_batch, print_batch, 
-		print_w, num_batch_lab, num_batch;
+		printer_text, param_text, printer_lab, param_lab,
+		clear_batch, print_batch, 
+		num_batch_lab, num_batch;
 
 static Widget	mag_lab;
 static Widget	size_lab;
@@ -66,10 +79,10 @@ static void	fit_page();
 
 static Position xposn, yposn;
 static String   prn_translations =
-        "<Message>WM_PROTOCOLS: DismissPrin()\n";
+        "<Message>WM_PROTOCOLS: DismissPrint()\n";
 static void     print_panel_dismiss(), do_clear_batch();
 static void	get_magnif();
-static XtCallbackProc update_mag();
+static void update_mag();
 void		do_print(), do_print_batch();
 
 /* callback list to keep track of magnification window */
@@ -89,11 +102,11 @@ String  print_translations =
 
 static XtActionsRec     prn_actions[] =
 {
-    {"DismissPrin", (XtActionProc) print_panel_dismiss},
-    {"dismiss", (XtActionProc) print_panel_dismiss},
-    {"print_batch", (XtActionProc) do_print_batch},
-    {"clear_batch", (XtActionProc) do_clear_batch},
-    {"print", (XtActionProc) do_print},
+    {"DismissPrint", (XtActionProc) print_panel_dismiss},
+    {"Dismiss", (XtActionProc) print_panel_dismiss},
+    {"PrintBatch", (XtActionProc) do_print_batch},
+    {"ClearBatch", (XtActionProc) do_clear_batch},
+    {"Print", (XtActionProc) do_print},
     {"UpdateMag", (XtActionProc) update_mag},
 };
 
@@ -119,6 +132,7 @@ do_print(w)
 	char	   *param_val;
 	char	    cmd[255],cmd2[255];
 	char	   *c1;
+	char	    backgrnd[10];
 
 	/* don't print if in the middle of drawing/editing */
 	if (check_action_on())
@@ -158,7 +172,9 @@ do_print(w)
 		    strcat(c1, cmd2);		/* append tail */
 		}
 	    }
-	    print_to_printer(printer_val, appres.magnification, cmd);
+	    /* make a #rrggbb string from the background color */
+	    make_rgb_string(export_background_color, backgrnd);
+	    print_to_printer(printer_val, backgrnd, appres.magnification, cmd);
 	}
 }
 
@@ -228,7 +244,7 @@ get_magnif()
 
 /* as the user types in a magnification, update the figure size */
 
-static XtCallbackProc
+static void
 update_mag(widget, item, event)
     Widget	    widget;
     XtPointer	   *item;
@@ -266,6 +282,7 @@ do_print_batch(w)
 	FILE	   *infp,*outfp;
 	char	    tmp_exp_file[32];
 	char	    str[255];
+	char	    backgrnd[10];
 
 	if (writing_batch || emptyfigure_msg(print_msg))
 		return;
@@ -288,7 +305,11 @@ do_print_batch(w)
 	/* update the figure size (magnification * bounding_box) */
 	print_update_figure_size();
 
-	print_to_file(tmp_exp_file, "ps", appres.magnification, 0, 0);
+	/* make a #rrggbb string from the background color */
+	make_rgb_string(export_background_color, backgrnd);
+
+	print_to_file(tmp_exp_file, "ps", appres.magnification, 0, 0, backgrnd,
+				NULL, FALSE, 0, FALSE);
 	put_msg("Appending to batch file \"%s\" (%s mode) ... done",
 		    batch_file, appres.landscape ? "LANDSCAPE" : "PORTRAIT");
 	app_flush();		/* make sure message gets displayed */
@@ -386,6 +407,8 @@ papersize_select(w, new_papersize, garbage)
     if (export_papersize_panel)
 	SetValues(export_papersize_panel);
     appres.papersize = papersize;
+    /* update the red line showing the new page size */
+    update_pageborder();
 }
 
 static void
@@ -419,6 +442,49 @@ multiple_select(w, new_multiple, garbage)
     }
 }
 
+/* user selected a background color from the menu */
+
+static void
+background_select(w, closure, call_data)
+    Widget	    w;
+    XtPointer	    closure, call_data;
+{
+    Pixel	    bgcolor, fgcolor;
+
+    /* get the colors from the color button just pressed */
+    FirstArg(XtNbackground, &bgcolor);
+    NextArg(XtNforeground, &fgcolor);
+    GetValues(w);
+
+    /* get the colorname from the color button and put it and the colors 
+       in the menu button */
+    FirstArg(XtNlabel, XtName(w));
+    NextArg(XtNbackground, bgcolor);
+    NextArg(XtNforeground, fgcolor);
+    SetValues(print_background_panel);
+    /* update the export panel too if it exists */
+    if (export_background_panel)
+	SetValues(export_background_panel);
+    export_background_color = (int)closure;
+
+    XtPopdown(background_menu);
+}
+
+static void
+printer_select(w, new_printer, garbage)
+    Widget	    w;
+    XtPointer	    new_printer, garbage;
+{
+    int printer = (int) new_printer;
+
+    /* put selected printer in the menu button */
+    FirstArg(XtNlabel, printer_names[printer]);
+    SetValues(printer_menu_button);
+    /* and in the printer text widget */
+    FirstArg(XtNstring, printer_names[printer]);
+    SetValues(printer_text);
+}
+
 /* update the figure size window */
 
 void
@@ -456,14 +522,23 @@ popup_print_panel(w)
 	SetValues(print_mag_text);
 	/* also the figure size (magnification * bounding_box) */
 	print_update_figure_size();
+	/* and the background color menu */
+	XtDestroyWidget(background_menu);
+	background_menu = make_color_popup_menu(print_background_panel,
+	    				"Background Color", background_select, 
+					False, True);
     } else {
 	create_print_panel(w);
     }
     XtPopup(print_popup, XtGrabNone);
+    /* now that the popup is realized, put in the name of the first printer */
+    if (printer_names[0] != NULL) {
+	FirstArg(XtNlabel, printer_names[0]);
+	SetValues(printer_menu_button);
+    }
     /* insure that the most recent colormap is installed */
     set_cmap(XtWindow(print_popup));
-    (void) XSetWMProtocols(XtDisplay(print_popup), XtWindow(print_popup),
-                           &wm_delete_window, 1);
+    (void) XSetWMProtocols(tool_d, XtWindow(print_popup), &wm_delete_window, 1);
     reset_cursor();
 
 }
@@ -479,17 +554,17 @@ create_print_panel(w)
 	char	    buf[50];
 	char	   *unit;
 	int	    ux,uy,lx,ly;
-	int	    i;
+	int	    i,len,maxl;
 	float	    mult;
+	XColor	    xcolor;
 
-	print_w = w;
-	XtTranslateCoords(w, (Position) 0, (Position) 0, &xposn, &yposn);
+	XtTranslateCoords(tool, (Position) 0, (Position) 0, &xposn, &yposn);
 
-	FirstArg(XtNx, xposn);
-	NextArg(XtNy, yposn + 50);
+	FirstArg(XtNx, xposn+50);
+	NextArg(XtNy, yposn+50);
 	NextArg(XtNtitle, "Xfig: Print menu");
 	NextArg(XtNcolormap, tool_cm);
-	print_popup = XtCreatePopupShell("print_menu",
+	print_popup = XtCreatePopupShell("print_popup",
 					 transientShellWidgetClass,
 					 tool, Args, ArgCount);
         XtOverrideTranslations(print_popup,
@@ -536,7 +611,7 @@ create_print_panel(w)
 	/* note: this was called "magnification" */
 	sprintf(buf, "%.1f", appres.magnification);
 	mag_spinner = MakeFloatSpinnerEntry(print_panel, &print_mag_text, "magnification",
-				image, mag_lab, mag_callback, buf, 0.0, 10000.0);
+				image, mag_lab, mag_callback, buf, 0.0, 10000.0, 1.0, 45);
 
 	/* we want to track typing here to update figure size label */
 
@@ -616,25 +691,26 @@ create_print_panel(w)
 	print_orient_panel = XtCreateManagedWidget(orient_items[appres.landscape],
 					     menuButtonWidgetClass,
 					     print_panel, Args, ArgCount);
-	orient_menu = make_popup_menu(orient_items, XtNumber(orient_items),
+	orient_menu = make_popup_menu(orient_items, XtNumber(orient_items), -1, "",
 				      print_orient_panel, orient_select);
 
-	FirstArg(XtNlabel, "   Justification");
+	FirstArg(XtNlabel, "Justification");
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
-	NextArg(XtNfromVert, print_orient_panel);
+	NextArg(XtNfromHoriz, print_orient_panel);
+	NextArg(XtNfromVert, print_papersize_panel);
 	just_lab = XtCreateManagedWidget("just_label", labelWidgetClass,
 					 print_panel, Args, ArgCount);
 
 	FirstArg(XtNlabel, just_items[appres.flushleft? 1 : 0]);
 	NextArg(XtNfromHoriz, just_lab);
-	NextArg(XtNfromVert, print_orient_panel);
+	NextArg(XtNfromVert, print_papersize_panel);
 	NextArg(XtNborderWidth, INTERNAL_BW);
 	NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
 	print_just_panel = XtCreateManagedWidget("justify",
 					   menuButtonWidgetClass,
 					   print_panel, Args, ArgCount);
-	just_menu = make_popup_menu(just_items, XtNumber(just_items),
+	just_menu = make_popup_menu(just_items, XtNumber(just_items), -1, "",
 				    print_just_panel, just_select);
 
 	/* multiple/single page */
@@ -654,13 +730,52 @@ create_print_panel(w)
 	print_multiple_panel = XtCreateManagedWidget("multiple_pages",
 					   menuButtonWidgetClass,
 					   print_panel, Args, ArgCount);
-	multiple_menu = make_popup_menu(multiple_pages, XtNumber(multiple_pages),
+	multiple_menu = make_popup_menu(multiple_pages, XtNumber(multiple_pages), -1, "",
 				    print_multiple_panel, multiple_select);
 
+	/* background color */
+
+	FirstArg(XtNlabel, "   Background");
+	NextArg(XtNjustify, XtJustifyLeft);
+	NextArg(XtNfromHoriz, print_multiple_panel);
+	NextArg(XtNfromVert, print_just_panel);
+	NextArg(XtNborderWidth, 0);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	NextArg(XtNright, XtChainLeft);
+	background_lab = XtCreateManagedWidget("background_label", labelWidgetClass,
+					 print_panel, Args, ArgCount);
+
+	/* put the canvas background colorname in to start */
+	set_color_name(export_background_color, buf);
+	FirstArg(XtNlabel, buf);
+	NextArg(XtNbackground, x_color(export_background_color));  /* set color of button */
+	NextArg(XtNfromHoriz, background_lab);
+	NextArg(XtNfromVert, print_just_panel);
+	NextArg(XtNresize, False);
+	NextArg(XtNwidth, 80);
+	NextArg(XtNborderWidth, INTERNAL_BW);
+	NextArg(XtNtop, XtChainTop);
+	NextArg(XtNbottom, XtChainTop);
+	NextArg(XtNleft, XtChainLeft);
+	print_background_panel = XtCreateManagedWidget("background",
+					   menuButtonWidgetClass,
+					   print_panel, Args, ArgCount);
+	/* now set foreground to contrasting color */
+	xcolor.pixel = x_color(export_background_color);
+	XQueryColor(tool_d, tool_cm, &xcolor);
+	pick_contrast(xcolor, print_background_panel);
+
+	/* make color menu */
+	background_menu = make_color_popup_menu(print_background_panel, 
+					"Background Color", background_select,
+					False, True);
+
 	/* printer name */
 
 	FirstArg(XtNlabel, "         Printer");
-	NextArg(XtNfromVert, print_multiple_panel);
+	NextArg(XtNfromVert, print_background_panel);
 	NextArg(XtNjustify, XtJustifyLeft);
 	NextArg(XtNborderWidth, 0);
 	printer_lab = XtCreateManagedWidget("printer_label", labelWidgetClass,
@@ -671,7 +786,7 @@ create_print_panel(w)
 	 */
 
 	FirstArg(XtNwidth, 100);
-	NextArg(XtNfromVert, print_multiple_panel);
+	NextArg(XtNfromVert, print_background_panel);
 	NextArg(XtNfromHoriz, printer_lab);
 	NextArg(XtNeditType, XawtextEdit);
 	NextArg(XtNinsertPosition, 0);
@@ -696,6 +811,32 @@ create_print_panel(w)
 			SetValues(printer_text);
 		}
 	}
+	/* parse /etc/printcap for printernames for the pull-down menu */
+	numprinters = parse_printcap(&printer_names);
+	/* find longest name */
+	maxl = 0;
+	for (i=0; i<numprinters; i++) {
+	    len=strlen(printer_names[i]);
+	    if (len > maxl) {
+		maxl = len;
+	    }
+	}
+	/* make string of blanks the length of the longest printer name */
+	buf[0] = '\0';
+	for (i=0; i<maxl; i++)
+	    strcat(buf," ");
+	if (numprinters > 0) {
+	    FirstArg(XtNlabel, buf);
+	    NextArg(XtNfromHoriz, printer_text);
+	    NextArg(XtNfromVert, print_background_panel);
+	    NextArg(XtNborderWidth, INTERNAL_BW);
+	    NextArg(XtNleftBitmap, menu_arrow);	/* use menu arrow for pull-down */
+	    printer_menu_button = XtCreateManagedWidget("printer_names",
+					   menuButtonWidgetClass,
+					   print_panel, Args, ArgCount);
+	    printer_menu = make_popup_menu(printer_names, numprinters, -1, "",
+				    printer_menu_button, printer_select);
+	}
 
 	FirstArg(XtNlabel, "Print Job Params");
 	NextArg(XtNfromVert, printer_text);
@@ -783,6 +924,7 @@ create_print_panel(w)
 	XtAddEventHandler(clear_batch, ButtonReleaseMask, (Boolean) 0,
 			  (XtEventHandler)do_clear_batch, (XtPointer) NULL);
 
+	/* install accelerators for the following functions */
 	XtInstallAccelerators(print_panel, dismiss);
 	XtInstallAccelerators(print_panel, print_batch);
 	XtInstallAccelerators(print_panel, clear_batch);
@@ -806,3 +948,68 @@ create_print_panel(w)
 	    }
 	}
 }
+
+/* if the users's sytem doesn't have an /etc/printcap file, this will return 0 */
+
+static int
+parse_printcap(names)
+char *names[];
+{
+    FILE   *printcap;
+    char    str[300];
+    int     i,j,len;
+    int     printers;
+    Boolean comment;
+
+    if ((printcap=fopen("/etc/printcap","r"))==NULL)
+	return 0;
+    printers = 0;
+    while (!feof(printcap)) {
+	if (fgets(str, sizeof(str), printcap) == NULL)
+		return printers;
+	len = strlen(str);
+	comment = False;
+	/* get rid of newline */
+	str[--len] = '\0';
+	/* check for comments */
+	for (i=0; i<len; i++) {
+	    if (str[i] == '#') {
+		comment = True;
+		break;
+	    }
+	    if (str[i] != ' ' && str[i] != '\t')
+		break;
+	}
+	/* skip comment */
+	if (comment)
+	    continue;
+	/* skip blank line */
+	if (i==len)
+	    continue;
+	/* get printer name */
+	for (j=0; j<len; j++) {
+	    if (str[j] == '|' || str[j] == ':')
+		break;
+	}
+	str[j] = '\0';
+	if ((names[printers] = malloc(j-i+1)) == NULL)
+	    return printers;
+	strncpy(names[printers],&str[i],j-i+1);
+	printers++;
+	for (j=len-1; j>0; j--) {
+	    if (str[j] == ' ' || str[j] == '\t')
+		continue;
+	    /* found the next entry, break */
+	    if (str[j] != '\\')
+		break;
+	    /* this line has \ at the end, read the next line and check it */
+	    if (fgets(str, sizeof(str), printcap) == NULL)
+		break;
+	    /* set length to ignore newline */
+	    len = strlen(str)-1;
+	    /* force loop to start over */
+	    j=len;
+	}
+    }
+    return printers;
+}
